class SQLiteMobxColumn {
    constructor(name, type, size) {
        this._name = name;
        this._type = type;
        this._size = size;
    }
    nullable(value = true) {
        this._nullable = value;
    }
    default(value) {
        this._default = value;
    }
    toSQL() {
        let sql = `${this._name} ${this._type}`;
        // Add column size
        if (this._size) {
            sql += `(${this._size})`;
        }
        // Add Nullable
        if (this._nullable) {
            sql += ' NULL';
        }
        else {
            sql += ' NOT NULL';
        }
        // SET Default value
        if (this._default) {
            sql += ` DEFAULT ${this._default}`;
        }
        // Add Finish comma
        return sql + ',\n';
    }
}
class SQLiteMobxTable {
    constructor(name) {
        this._columns = {};
        this._name = name;
    }
    primary(param = 'id', size = 11) {
        const type = `INTEGER(${size}) PRIMARY KEY`;
        const column = new SQLiteMobxColumn(param, type, 0);
        column.nullable(false);
        return (this._columns[param] = column);
    }
    string(param, size = 50) {
        return (this._columns[param] = new SQLiteMobxColumn(param, 'VARCHAR', size));
    }
    integer(param, size = 11) {
        return (this._columns[param] = new SQLiteMobxColumn(param, 'INTEGER', size));
    }
    boolean(param, size = 0) {
        return (this._columns[param] = new SQLiteMobxColumn(param, 'BOOLEAN', size));
    }
    ifNotExists(value = true) {
        this._ifNotExists = value;
    }
    toSQL() {
        let sql = 'CREATE TABLE';
        // Add IF NOT EXISTS
        if (this._ifNotExists) {
            sql += ' IF NOT EXISTS';
        }
        // Set Column name and starts block
        sql += ` ${this._name} (\n`;
        // Add Column list
        for (const key in this._columns) {
            sql += this._columns[key].toSQL();
        }
        // Remove last comma and finish block
        return sql.replace(/,\n$/g, '\n);');
    }
}
class SQLiteMobxSchema {
    static create(name, actioner) {
        const table = new SQLiteMobxTable(name);
        table.primary();
        actioner(table);
    }
}
export { SQLiteMobxTable, SQLiteMobxColumn };
export default SQLiteMobxSchema;
//# sourceMappingURL=SQLiteMobxSchema.js.map